#!/usr/bin/perl

# Author : Robert Vojta (robert.vojta@mandrake.cz)
#
# Contributors : 
#
# Copyright (C) 2004 Robert Vojta (robert.vojta@mandrake.cz)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#
# Module for /etc/sysconfig/desktop handling
#
# You can use desktop, DESKTOP, displaymanager, DISPLAYMANAGER
# 
# Example:
#
# use Libconf::System::Desktop;
# my $cfg = new Libconf::System::Desktop;
# $t->{general}->{desktop} = "GNOME";
# $t->{general}->{displaymanager} = "KDE"; 
# $t->writeConf;
#

package Libconf::System::Desktop::General;
use strict;
use vars qw(@ISA);
use Libconf qw(:helpers);
@ISA = qw(Libconf);

sub new {
    my ($class) = @_;
    tie my %wrapper, 'Libconf::System::Desktop::GeneralWrapper';
    bless \%wrapper, $class;
}

sub readConf {
    my ($self) = @_;
    $self->{_desktop}->readConf();
}

sub writeConf {
    my ($self) = @_;
    $self->{_desktop}->writeConf();
    $self->{_autologin}->writeConf();
}

package Libconf::System::Desktop::GeneralWrapper;
use Libconf qw(:helpers);
use Libconf::Glueconf::Generic::Shell;
use Libconf::System::Users;

sub debug { Libconf::debug(@_) }
my $sys_users = new Libconf::System::Users();

sub TIEHASH {
    my ($pkg) = @_;
    debug();
    my $self = {
	desktop => new Libconf::Glueconf::Generic::Shell('/etc/sysconfig/desktop'),
	autologin => new Libconf::Glueconf::Generic::Shell('/etc/sysconfig/autologin'),
    };
    bless $self, $pkg;
    $self
}

sub CLEAR {
    my ($self) = @_;
    debug();
    $self->{desktop} = {};
}

sub DELETE {
    my ($self, $key) = @_;
    debug("key: $key");
    delete $self->{desktop}->{$key};
}

sub FIRSTKEY {
    my ($self) = @_;
    debug();
    'desktop';
}

sub EXISTS {
    my ($self, $key) = @_;
    debug("key : $key");
    Libconf::member($key, qw(desktop displaymanager));
}

sub NEXTKEY {
    my ($self, $lastkey) = @_;
    debug("lastkey : $lastkey");
    $lastkey eq 'desktop' and return 'displaymanager';
    $lastkey eq 'displaymanager' and return 'autologin';
    undef;
}

sub STORE {
    my ($self, $key, $value) = @_;
    debug("key : $key - value : $value");
    if (Libconf::member($key, qw(desktop displaymanager))) {
        $key = uc($key);
        $self->{desktop}->{$key} = $value;
    } elsif ($key eq 'autologin') {
        # setting autologin
        $self->{autologin}->{AUTOLOGIN} = bool2yesno($value);
	$self->{autologin}->{EXEC} = '/usr/X11R6/bin/startx.autologin';

        if (defined $value && $value ne '') {
            # This is only as demo purpose, this should be done by validation system.
            $sys_users->readConf();
            my @user_list = $sys_users->getUsersList(sub { $_[0]->{UID} >= 500 });
            Libconf::member($value, @user_list) or warn qq("$value" is not a valid user \n);
        }
	$self->{autologin}->{USER} = $value;
    }
}

sub FETCH {
    my ($self, $key) = @_;
    debug("key : $key");
    $key =~ /^_(.*)$/ and return $self->{$1};
    if (Libconf::member($key, qw(desktop displaymanager))) {
        $key = uc($key);
        return $self->{desktop}->{$key};
    } elsif ($key eq 'autologin') {
        return $self->{autologin}->{USER};
    }
}

1;
