#!/usr/bin/perl

# Author : Damien KROTKINE (damien@tuxfamily.org)
#
# Contributors : 
#
# Copyright (C) 2002 Damien KROTKINE (damien@tuxfamily.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package Libconf::System::Network;
use strict;
use vars qw(@ISA);
use Libconf;
use Libconf::Glueconf::Generic::Shell;
use Libconf::Glueconf::Networking::Resolv;
use Libconf::Glueconf::Networking::Hosts;
use Libconf::Glueconf::Networking::Network;
use Libconf::System::DirectoryWrapper;
use Libconf::System::Network::Proxy;

@ISA = qw(Libconf);

sub new {
    my ($pkg) = @_;
    tie my %h_interfaces, 'Libconf::System::DirectoryWrapper', '/etc/sysconfig/network-scripts', 'ifcfg-*', 'Networking::Ifconfig';
    bless {
           general => new Libconf::Glueconf::Networking::Network('/etc/sysconfig/network'),
           dns => new Libconf::Glueconf::Networking::Resolv('/etc/resolv.conf'),
           interfaces => \%h_interfaces,
           hosts => new Libconf::Glueconf::Networking::Hosts('/etc/hosts'),
           proxy => new Libconf::System::Network::Proxy(),
          }, $pkg;
}

sub readConf {
    my ($self) = @_;
    $self->{$_}->readConf foreach qw(general dns hosts proxy);
    $_->readConf foreach values %{$self->{interfaces}}
}

sub writeConf {
    my ($self) = @_;
    $self->{$_}->writeConf foreach qw(general dns hosts proxy);
    $_->writeConf foreach values %{$self->{interfaces}}
}

1;
