package Libconf::System::Network::Proxy;
use strict;
use vars qw(@ISA);
use Libconf;
@ISA = qw(Libconf);

sub new {
    my ($class) = @_;
    tie my %wrapper, 'Libconf::System::Network::ProxyWrapper';
    bless \%wrapper, $class;
}

sub readConf {
    my ($self) = @_;
    $self->{_proxy_sh}->readConf();
    $self->{_proxy_csh}->readConf();
}

sub writeConf {
    my ($self) = @_;
    $self->{_proxy_sh}->writeConf();
    $self->{_proxy_csh}->writeConf();
}

package Libconf::System::Network::ProxyWrapper;
use Libconf::Glueconf::Generic::ExportedShell;
use Libconf::Glueconf::Generic::ExportedCShell;

sub debug { Libconf::debug(@_) }

my $PROXYSH_FILENAME = '/etc/profile.d/proxy.sh';
my $PROXYCSH_FILENAME = '/etc/profile.d/proxy.csh';
my $KEY_HTTP = 'http_proxy';
my $KEY_FTP = 'ftp_proxy';
my @KEYS = ($KEY_HTTP, $KEY_FTP);

sub synchronize {
    my ($self) = @_;
    foreach my $key (@KEYS) {
        if ($self->{proxy_sh}->{$key} ne $self->{proxy_csh}->{$key}) {
            warn "Libconf::System::Network::Proxy - synchronizing $PROXYSH_FILENAME and $PROXYCSH_FILENAME :\n";
            warn "Libconf::System::Network::Proxy - $key differ in $PROXYCSH_FILENAME, correcting it from $PROXYSH_FILENAME\n";
            $self->{proxy_csh}->{$key} = $self->{proxy_sh}->{$key};
        }
        if ($self->{proxy_csh}->{$key} ne $self->{proxy_sh}->{$key}) {
            warn "Libconf::System::Network::Proxy - synchronizing $PROXYSH_FILENAME and $PROXYCSH_FILENAME :\n";
            warn "Libconf::System::Network::Proxy - $key differ in $PROXYSH_FILENAME, correcting it from $PROXYCSH_FILENAME\n";
            $self->{proxy_sh}->{$key} = $self->{proxy_csh}->{$key};
        }
    }
}

sub TIEHASH {
    my ($pkg) = @_;
    debug();
    my $self = {
                proxy_sh => new Libconf::Glueconf::Generic::ExportedShell($PROXYSH_FILENAME),
                proxy_csh => new Libconf::Glueconf::Generic::ExportedCShell($PROXYCSH_FILENAME),
               };
    bless $self, $pkg;
    $self->synchronize();
    $self
}

sub CLEAR {
    my ($self) = @_;
    debug();
    $self->{proxy_csh} = {};
    $self->{proxy_sh} = {};
}

sub DELETE {
    my ($self, $key) = @_;
    debug("key: $key");
    delete $self->{proxy_csh}->{$key};
    delete $self->{proxy_sh}->{$key};
}

sub FIRSTKEY {
    my ($self) = @_;
    debug();
    'http';
}

sub EXISTS {
    my ($self, $key) = @_;
    debug("key : $key");
    Libconf::member($key, qw(http ftp));
}

sub NEXTKEY {
    my ($self, $lastkey) = @_;
    debug("lastkey : $lastkey");
    $lastkey eq 'http' and return 'ftp';
    undef;
}

sub STORE {
    my ($self, $key, $value) = @_;
    debug("key : $key - value : $value");
    $self->synchronize();
    Libconf::member($key, qw(http ftp)) and $key .= '_proxy';
    $self->{proxy_sh}->{$key} = $value;
    $self->{proxy_csh}->{$key} = $value;
}

sub FETCH {
    my ($self, $key) = @_;
    debug("key : $key");
    $key =~ /^_(.*)$/ and return $self->{$1};
    $self->synchronize();
    Libconf::member($key, qw(http ftp)) and $key .= '_proxy';
    $self->{proxy_sh}->{$key};
}

1;
